@@ -35,6 +35,10 @@
 #include "glaze/net/http_router.hpp"
 #include "glaze/util/parse.hpp"
 
+#ifdef GLZ_ENABLE_SSL
+#include <asio/ssl.hpp>
+#endif
+
 namespace glz
 {
    // WebSocket opcode constants
@@ -268,6 +272,40 @@ namespace glz
       virtual void force_close() = 0;
    };
 
+   // Type-erased interface for WebSocket connections.
+   // Allows handlers to work with both TCP (ws://) and SSL (wss://) WebSocket connections.
+   //
+   // NOTE: Handler types now use websocket_connection_interface instead of the concrete
+   // websocket_connection<tcp::socket>. Using `auto` in lambdas still works:
+   //   ws_server->on_message([](auto conn, std::string_view msg, ws_opcode op) {
+   //      conn->send_text("reply");  // interface methods work fine
+   //   });
+   // Only code that explicitly typed the concrete type or accessed internal members
+   // (like socket_) needs to be updated.
+   struct websocket_connection_interface : public closeable_connection
+   {
+      // Send operations
+      virtual void send_text(std::string_view message) = 0;
+      virtual void send_binary(std::string_view message) = 0;
+      virtual void send_ping(std::string_view payload = {}) = 0;
+      virtual void send_pong(std::string_view payload = {}) = 0;
+
+      // Close the connection
+      virtual void close(ws_close_code code = ws_close_code::normal, std::string_view reason = {}) = 0;
+
+      // Connection info
+      virtual std::string remote_address() const = 0;
+      virtual uint16_t remote_port() const = 0;
+
+      // User data
+      virtual void set_user_data(std::shared_ptr<void> data) = 0;
+      virtual std::shared_ptr<void> get_user_data() const = 0;
+
+      // Close info (valid after connection is closed)
+      virtual ws_close_code get_close_code() const = 0;
+      virtual std::string_view get_close_reason() const = 0;
+   };
+
    // WebSocket server class
    //
    // Message Handler Lifetime:
@@ -283,14 +321,14 @@ namespace glz
    //
    struct websocket_server
    {
+      // Handler types use websocket_connection_interface for type erasure.
+      // This allows handlers to work with both TCP (ws://) and SSL (wss://) connections.
       using message_handler =
-         std::function<void(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>>, std::string_view, ws_opcode)>;
-      using close_handler = std::function<void(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>>,
-                                               ws_close_code, std::string_view)>;
-      using error_handler =
-         std::function<void(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>>, std::error_code)>;
-      using open_handler =
-         std::function<void(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>>, const request&)>;
+         std::function<void(std::shared_ptr<websocket_connection_interface>, std::string_view, ws_opcode)>;
+      using close_handler =
+         std::function<void(std::shared_ptr<websocket_connection_interface>, ws_close_code, std::string_view)>;
+      using error_handler = std::function<void(std::shared_ptr<websocket_connection_interface>, std::error_code)>;
+      using open_handler = std::function<void(std::shared_ptr<websocket_connection_interface>, const request&)>;
       using validate_handler = std::function<bool(const request&)>;
 
       websocket_server() = default;
@@ -344,23 +382,26 @@ namespace glz
       // Set connection validator
       inline void on_validate(validate_handler handler) { validate_handler_ = std::move(handler); }
 
-      // Internal methods called by websocket_connection
-      inline void notify_open(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>> conn, const request& req)
+      // Internal methods called by websocket_connection (templated to accept any socket type)
+      template <typename SocketType>
+      inline void notify_open(std::shared_ptr<websocket_connection<SocketType>> conn, const request& req)
       {
          if (open_handler_) {
             open_handler_(conn, req);
          }
       }
 
-      inline void notify_message(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>> conn,
-                                 std::string_view message, ws_opcode opcode)
+      template <typename SocketType>
+      inline void notify_message(std::shared_ptr<websocket_connection<SocketType>> conn, std::string_view message,
+                                 ws_opcode opcode)
       {
          if (message_handler_) {
             message_handler_(conn, message, opcode);
          }
       }
 
-      inline void notify_close(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>> conn, ws_close_code code,
+      template <typename SocketType>
+      inline void notify_close(std::shared_ptr<websocket_connection<SocketType>> conn, ws_close_code code,
                                std::string_view reason)
       {
          if (close_handler_) {
@@ -368,7 +409,8 @@ namespace glz
          }
       }
 
-      inline void notify_error(std::shared_ptr<websocket_connection<asio::ip::tcp::socket>> conn, std::error_code ec)
+      template <typename SocketType>
+      inline void notify_error(std::shared_ptr<websocket_connection<SocketType>> conn, std::error_code ec)
       {
          if (error_handler_) {
             error_handler_(conn, ec);
@@ -406,7 +448,7 @@ namespace glz
    // is only valid for the duration of the callback. Copy the data if you need to
    // retain it beyond the callback scope.
    template <typename SocketType>
-   struct websocket_connection : public closeable_connection,
+   struct websocket_connection : public websocket_connection_interface,
                                  public std::enable_shared_from_this<websocket_connection<SocketType>>
    {
      public:
@@ -416,13 +458,7 @@ namespace glz
       {
          // Try to get remote endpoint, but don't fail if it's not available yet
          try {
-            if constexpr (std::is_same_v<SocketType, asio::ip::tcp::socket>) {
-               remote_endpoint_ = socket_->remote_endpoint();
-            }
-            else {
-               // For SSL sockets, get the endpoint from the lowest layer
-               remote_endpoint_ = socket_->lowest_layer().remote_endpoint();
-            }
+            remote_endpoint_ = socket_->lowest_layer().remote_endpoint();
          }
          catch (...) {
             // Endpoint not available yet (e.g., for client mode before connection)
@@ -452,22 +488,22 @@ namespace glz
       inline void start(const request& req) { perform_handshake(req); }
 
       // Send a text message
-      inline void send_text(std::string_view message) { send_frame(ws_opcode::text, message); }
+      void send_text(std::string_view message) override { send_frame(ws_opcode::text, message); }
 
       // Send a binary message
-      inline void send_binary(std::string_view message) { send_frame(ws_opcode::binary, message); }
+      void send_binary(std::string_view message) override { send_frame(ws_opcode::binary, message); }
 
       // Send a ping frame
-      inline void send_ping(std::string_view payload = {}) { send_frame(ws_opcode::ping, payload); }
+      void send_ping(std::string_view payload = {}) override { send_frame(ws_opcode::ping, payload); }
 
       // Send a pong frame
-      inline void send_pong(std::string_view payload = {}) { send_frame(ws_opcode::pong, payload); }
+      void send_pong(std::string_view payload = {}) override { send_frame(ws_opcode::pong, payload); }
 
       // Close the connection
       // RFC 6455 Section 7.1.2: "To Start the WebSocket Closing Handshake with a status code,
       // an endpoint MUST send a Close control frame"
       // https://datatracker.ietf.org/doc/html/rfc6455#section-7.1.2
-      inline void close(ws_close_code code = ws_close_code::normal, std::string_view reason = {})
+      void close(ws_close_code code = ws_close_code::normal, std::string_view reason = {}) override
       {
          // Atomically check and set is_closing_ to prevent race conditions
          bool expected = false;
@@ -506,25 +542,25 @@ namespace glz
             std::lock_guard<std::mutex> lock(socket_op_mutex_);
             if (socket_) {
                asio::error_code ec;
-               socket_->cancel(ec); // Cancel pending operations first
-               socket_->close(ec);
+               socket_->lowest_layer().cancel(ec);
+               socket_->lowest_layer().close(ec);
                socket_.reset();
             }
          }
       }
 
       // Get remote endpoint information
-      inline std::string remote_address() const { return remote_endpoint_.address().to_string(); }
+      std::string remote_address() const override { return remote_endpoint_.address().to_string(); }
 
-      inline uint16_t remote_port() const { return remote_endpoint_.port(); }
+      uint16_t remote_port() const override { return remote_endpoint_.port(); }
 
       // Set user data
-      inline void set_user_data(std::shared_ptr<void> data) { user_data_ = data; }
-      inline std::shared_ptr<void> get_user_data() const { return user_data_; }
+      void set_user_data(std::shared_ptr<void> data) override { user_data_ = data; }
+      std::shared_ptr<void> get_user_data() const override { return user_data_; }
 
       // Get close code and reason (valid after connection is closed)
-      inline ws_close_code get_close_code() const { return close_code_; }
-      inline std::string_view get_close_reason() const { return close_reason_; }
+      ws_close_code get_close_code() const override { return close_code_; }
+      std::string_view get_close_reason() const override { return close_reason_; }
 
       // Inject initial data read during handshake
       inline void set_initial_data(std::string_view data)
@@ -598,7 +634,7 @@ namespace glz
          // Capture socket locally to prevent race condition
          auto socket = socket_;
          // Check both pointer validity AND that socket is still open
-         if (!socket || !socket->is_open()) {
+         if (!socket || !socket->lowest_layer().is_open()) {
             return;
          }
 
@@ -635,12 +671,7 @@ namespace glz
             // Only call error handlers if not already closed (prevents calling handlers
             // after force_close() when user's captured references may be invalid)
             if (!closed_.load()) {
-               if (auto srv = server_.lock()) {
-                  srv->notify_error(this->shared_from_this(), ec);
-               }
-               else if (client_error_handler_) {
-                  client_error_handler_(ec);
-               }
+               dispatch_error(ec);
             }
             // Close the connection after a read error (connection is likely broken)
             do_close();
@@ -774,12 +805,7 @@ namespace glz
                // Only call message handlers if not closed (prevents calling handlers
                // after force_close() when user's captured references may be invalid)
                if (!closed_.load()) {
-                  if (auto srv = server_.lock()) {
-                     srv->notify_message(this->shared_from_this(), message_view, opcode);
-                  }
-                  else if (client_message_handler_) {
-                     client_message_handler_(message_view, opcode);
-                  }
+                  dispatch_message(message_view, opcode);
                }
             }
             else {
@@ -819,12 +845,7 @@ namespace glz
                // Only call message handlers if not closed (prevents calling handlers
                // after force_close() when user's captured references may be invalid)
                if (!closed_.load()) {
-                  if (auto srv = server_.lock()) {
-                     srv->notify_message(this->shared_from_this(), message_view, current_opcode_);
-                  }
-                  else if (client_message_handler_) {
-                     client_message_handler_(message_view, current_opcode_);
-                  }
+                  dispatch_message(message_view, current_opcode_);
                }
                message_buffer_.clear();
                current_opcode_ = ws_opcode::continuation;
@@ -966,7 +987,7 @@ namespace glz
          // Check both pointer validity AND that socket is still open.
          // The socket could be closed (is_open() == false) while the shared_ptr is still valid.
          // Calling async_write on a closed socket crashes in ASIO's reactor.
-         if (!socket || !socket->is_open()) {
+         if (!socket || !socket->lowest_layer().is_open()) {
             // Socket was closed externally - ensure close callback fires if pending
             bool had_close_pending = false;
             {
@@ -1000,12 +1021,7 @@ namespace glz
                                  // Only call error handlers if not already closed (prevents calling handlers
                                  // after force_close() when user's captured references may be invalid)
                                  if (!self->closed_.load()) {
-                                    if (auto srv = self->server_.lock()) {
-                                       srv->notify_error(self, ec);
-                                    }
-                                    else if (self->client_error_handler_) {
-                                       self->client_error_handler_(ec);
-                                    }
+                                    self->dispatch_error(ec);
                                  }
                                  // Close the connection after a write error (connection is likely broken)
                                  self->do_close();
@@ -1029,7 +1045,7 @@ namespace glz
          auto socket = socket_;
          if (socket) {
             asio::error_code ec;
-            socket->shutdown(asio::ip::tcp::socket::shutdown_send, ec);
+            socket->lowest_layer().shutdown(asio::ip::tcp::socket::shutdown_send, ec);
             // Ignore shutdown errors - socket may already be closed
          }
          do_close();
@@ -1124,6 +1140,42 @@ namespace glz
          }
       }
 
+      // Helper to dispatch error notifications to server or client handler
+      void dispatch_error(std::error_code ec)
+      {
+         if (auto srv = server_.lock()) {
+            srv->notify_error(this->shared_from_this(), ec);
+            return;
+         }
+         if (client_error_handler_) {
+            client_error_handler_(ec);
+         }
+      }
+
+      // Helper to dispatch message notifications to server or client handler
+      void dispatch_message(std::string_view message, ws_opcode opcode)
+      {
+         if (auto srv = server_.lock()) {
+            srv->notify_message(this->shared_from_this(), message, opcode);
+            return;
+         }
+         if (client_message_handler_) {
+            client_message_handler_(message, opcode);
+         }
+      }
+
+      // Helper to dispatch close notifications to server or client handler
+      void dispatch_close()
+      {
+         if (auto srv = server_.lock()) {
+            srv->notify_close(this->shared_from_this(), close_code_, close_reason_);
+            return;
+         }
+         if (client_close_handler_) {
+            client_close_handler_(close_code_, close_reason_);
+         }
+      }
+
       inline void do_close()
       {
          // Ensure we only close once and call on_close once
@@ -1135,12 +1187,7 @@ namespace glz
          // Synchronize with force_close() to prevent calling handlers with dangling references
          {
             std::lock_guard<std::mutex> lock(close_handler_mutex_);
-            if (auto srv = server_.lock()) {
-               srv->notify_close(this->shared_from_this(), close_code_, close_reason_);
-            }
-            else if (client_close_handler_) {
-               client_close_handler_(close_code_, close_reason_);
-            }
+            dispatch_close();
          }
 
          // Hold socket_op_mutex_ while closing to prevent race with async_read_some/async_write
@@ -1149,8 +1196,8 @@ namespace glz
             std::lock_guard<std::mutex> lock(socket_op_mutex_);
             if (socket_) {
                asio::error_code ec;
-               socket_->cancel(ec); // Cancel pending operations first
-               socket_->close(ec);
+               socket_->lowest_layer().cancel(ec);
+               socket_->lowest_layer().close(ec);
                socket_.reset(); // Reset pointer so subsequent checks know socket is closed
             }
          }
@@ -1232,7 +1279,7 @@ namespace glz
 
          // Re-check under lock since socket may have been closed while waiting for mutex
          auto socket = socket_;
-         if (!socket || !socket->is_open()) {
+         if (!socket || !socket->lowest_layer().is_open()) {
             return;
          }
 
